#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include "time.h"
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Tone32.h>


#define BUZZER_PIN 16
#define BUZZER_CHANNEL 0
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
#define OLED_RESET     4 // Reset pin # (or -1 if sharing Arduino reset pin)

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);
HTTPClient http;

const char* ssid = "SSID";
const char* pass = "PASSWORD";
const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = 19800;
const int   daylightOffset_sec = 0;
char todayDate[11];
char *center_id[] = {"618085","570517","622686","630476","692991","608233"}; //Enter center IDs separated by commas
int centerCount = 6; //Enter the number of centers

void setup() {
  Serial.begin(115200);
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.display();
  display.clearDisplay();
  WiFi.begin(ssid, pass);
  Serial.print("Connecting to Wifi");
  while(WiFi.status() != WL_CONNECTED){
    Serial.print(".");
    delay(500);
  }
  Serial.println("\nConnected to WiFI network");
  Serial.println(WiFi.localIP());
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  printLocalTime();
}

void loop() {
  for( int k = 0; k < centerCount; k++){
    display.clearDisplay();
    display.setTextColor(SSD1306_WHITE);
    Serial.println("Fetching center " + String(center_id[k]));
  if(WiFi.status() == WL_CONNECTED){
    http.begin("https://cdn-api.co-vin.in/api/v2/appointment/sessions/public/calendarByCenter?center_id=" + String(center_id[k]) + "&date=" + String(todayDate));
    http.GET();
    DynamicJsonBuffer jb;
    JsonObject& obj = jb.parseObject(http.getStream());
    
    if (!obj.success()){
      Serial.println("parseObject() failed");
      return;
    }
    display.setTextSize(1);
    display.setCursor(0,0);
    String centerName = obj["centers"]["name"];
    Serial.println("Center: " + centerName);
    display.println(centerName);
    int sessionCount = obj["centers"]["sessions"].size();
    for(int j = 0; j < sessionCount; j++){
      String sessionDate = obj["centers"]["sessions"][j]["date"];
      String sessionCap = obj["centers"]["sessions"][j]["available_capacity"];
      String age_limit = obj["centers"]["sessions"][j]["min_age_limit"];
      String vaccine = obj["centers"]["sessions"][j]["vaccine"];
      Serial.println(vaccine + " - " + sessionCap + " slots available on " + sessionDate + " for age " + age_limit + "+" );
      Serial.println("");
      if(age_limit == "18"){
        display.println("");
        display.println(sessionDate);
        display.setTextSize(3);
        display.setCursor(50, 40);
        display.print(sessionCap);
        display.display();
        if(sessionCap != "0"){
          tone(BUZZER_PIN, NOTE_C4, 5000, BUZZER_CHANNEL);
          noTone(BUZZER_PIN, BUZZER_CHANNEL);
        }
        delay(2000);
      }
    }
  Serial.println("-----------------------------------------");
  }
  else{
    Serial.println("Connection lost");
  }
  display.display();
  delay(2000);
 }
}

void printLocalTime(){
  struct tm timeinfo;
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
    return;
  }
  strftime(todayDate, 11, "%d-%m-%Y", &timeinfo);
  Serial.println(todayDate);
}
